package cryptoUtils

import (
	"crypto/rsa"
	"crypto/tls"
	"crypto/x509"
	"encoding/pem"
	"log"
	"os"

	"software.sslmate.com/src/go-pkcs12"
)

type KeyStore struct {
	cert        *x509.Certificate
	caCertChain []*x509.Certificate
	privKey     *rsa.PrivateKey
}

func (k KeyStore) GetCert() *x509.Certificate {
	return k.cert
}

func (k KeyStore) GetCACertChain() []*x509.Certificate {
	return k.caCertChain
}

func (k KeyStore) GetPrivKey() *rsa.PrivateKey {
	return k.privKey
}

func ExtractAllOIDValues(cert *x509.Certificate) map[string]string {
	oidValueMap := make(map[string]string)
	for _, name := range cert.Subject.Names {
		oid := name.Type.String()
		value := name.Value.(string)
		oidValueMap[oid] = value
	}
	return oidValueMap
}

func LoadKeyStore(keyStorePath string, password string) KeyStore {

	var privKey *rsa.PrivateKey

	certFile, err := os.ReadFile(keyStorePath)
	if err != nil {
		log.Panicln("Provided certificate couldn't be opened")
	}

	privKeyInterface, cert, caCerts, err := pkcs12.DecodeChain(certFile, password)
	if err != nil {
		log.Panicln("PKCS12 key store couldn't be decoded")
	}
	privKey, ok := privKeyInterface.(*rsa.PrivateKey)
	if !ok {
		log.Panicln("Failed to convert private key to RSA private key")
	}

	if err := privKey.Validate(); err != nil {
		log.Panicln("Private key is not valid")
	}
	return KeyStore{cert: cert, caCertChain: caCerts, privKey: privKey}
}

func (k *KeyStore) GetTLSConfig() *tls.Config {

	certPEM := pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: k.cert.Raw})
	privKeyPEM := pem.EncodeToMemory(&pem.Block{Type: "RSA PRIVATE KEY", Bytes: x509.MarshalPKCS1PrivateKey(k.privKey)})
	certificate, err := tls.X509KeyPair(certPEM, privKeyPEM)
	if err != nil {
		log.Panicln("Could not load certificate and privkey to TLS", err)
	}

	config := &tls.Config{
		Certificates: []tls.Certificate{certificate},
	}
	return config
}

func (k *KeyStore) GetTLSConfigServer() *tls.Config {
	config := k.GetTLSConfig()

	//Add the CA certificate chain to a CertPool
	caCertPool := x509.NewCertPool()
	for _, caCert := range k.caCertChain {
		caCertPool.AddCert(caCert)
	}
	config.ClientCAs = caCertPool
	config.ClientAuth = tls.RequireAndVerifyClientCert

	return config
}

func (k *KeyStore) GetTLSConfigClient() *tls.Config {
	config := k.GetTLSConfig()

	//Add the CA certificate chain to a CertPool
	caCertPool := x509.NewCertPool()
	for _, caCert := range k.caCertChain {
		caCertPool.AddCert(caCert)
	}
	config.RootCAs = caCertPool

	//TODO: FIX THE VERIFICATION OF THE SERVER
	//config.ServerName = "SERVER"

	return config
}

func (k KeyStore) EncryptMessageContent(peerPubKey rsa.PublicKey, content []byte) []byte {
	// Digital envolope
	return nil
}
